// ==UserScript==
// @name         Fix ADS checkin comments in discussion and history of workitems
// @version      0.11
// @author       Tobias Sachs
//  ... in @match replace "ads" with the url of you Azure DevOps Server
// @match        https://ads/*
// @updateURL    https://src.twobees.de/?p=tampermonkeyscripts.git;a=blob_plain;f=AzureDevOpsCommentEnhancer.user.js;hb=HEAD
// @downloadURL  https://src.twobees.de/?p=tampermonkeyscripts.git;a=blob_plain;f=AzureDevOpsCommentEnhancer.user.js;hb=HEAD
// @grant        none
// @description
// ==/UserScript==

// 0.11: wait longer, to make sure all(?) data is loaded when working over slow connections
// 0.10: #Bugnumber to links, highlight comments for customors in checkins
// 0.09: update download/update URLs
// 0.08: fix typos/formatting
// 0.07: fix work item tampering
// 0.06: Add link to Changeset in diff view

/* jshint esversion:6 */
(function() {
    'use strict';
    let timerId;

    let fixWorkitems = () => {
        let found = document.getElementsByClassName("comment-content");
        fixCommentContents(found);

        found = document.getElementsByClassName("history-item-comment");
        fixCommentContents(found);

        console.debug("observe...");
    };

    let fixCommentContents = (items) => {
        if (items === null || items === undefined || items.length === 0) {
            return;
        }
        console.info("fixing '" + items.length +"' comments.");
        for (var i = 0; i < items.length; i++) {
            let el = items[i];
            let html = el.innerHTML;
            if (html.startsWith("Associated")) {
                html = html.replace(/(Associated with changeset )(\d*):/,
                    "<b>$1<a href='/HeBa/Entwicklung/_versionControl/changeset/$2'>$2</a></b>:<br />");
                html = html.replace(/#(\d+)/g, "<a href='/HeBa/Entwicklung/_versionControl/changeset/$1'>#$1</a>");
                html = html.replace(/\n/gi, "<br />");
                html = html.replace(/(@@CUST.*)/s, "<br><span style=\"color: green; font-style:italic;\">$1</span>");
                el.innerHTML = html;
            }
        }
    };

    let fixVersionControl = () => {
        let elToFix;
        let found = document.getElementsByClassName("changeset-version")[0];
        if (!found) {
            return;
        }

        // if opened from email notification, it is the first span in div "changeset-version"
        elToFix = found.querySelector("span");

        if (!elToFix) {
            return;
        }

        elToFix.innerHTML = elToFix.innerHTML
            .replace(/(Changeset )(\d+)/,
                "$1<a href='/HeBa/Entwicklung/_versionControl/changeset/$2'>$2</a>");
    };

    let fixit = () => {
        if (timerId) {
            console.debug("fixit timerreset...");
            clearTimeout(timerId);
        }

        observer.disconnect();

        timerId = setTimeout(function() {
            timerId = undefined;

            let url = window.location.href;

            if (url.includes("/_versionControl")) {
                fixVersionControl();
            }
            else {
                // if (url.includes("/_workitems")){
                // does not work since workitems are often shown in
                // dialogs on random pages
                fixWorkitems();
            }

            // keep watching for changes
            observer.observe(document, { subtree: true, childList: true, characterData: true });
        }, 600);
    };

    const observer = new MutationObserver(function() {
        console.debug('observer was triggered...');
        fixit();
    });

    fixit();

})();

